##  Author: Stefan Evert
## Purpose: Sample evaluation for Free Association Task 1

## load official test data
Test <- read.delim("FA/discrimination_test.tbl")

## load model output (here for the FOO model)
Model <- read.delim("foo_output/task1.tbl")

## FOO only makes a coarse distinction between "FIRST" and "OTHER", so map test data
##  - if your model attempts the 3-way classification, skip this step
Test$type <- factor(ifelse(Test$type == "FIRST", "FIRST", "OTHER"))

## combine the test data and model output, matching on cue and target
##  - if your model output file uses different column names, change the by.y argument
All <- merge(Test, Model, by.x=c("cue","target"), by.y=c("cue","target"))
stopifnot(nrow(All) == nrow(Test)) # abort if model output doesn't have classifications for all cue-target pairs

## classification accuracy
accuracy <- 100 * sum(All$type == All$model.class) / nrow(All)
baseline <- 100 * max(table(All$type)) / nrow(All) # baseline: assign all pairs to largest category
cat("Classification accuracy: ", round(accuracy,2), "% (over baseline of ", round(baseline,2), "%)\n", sep="")

## compute precision and recall separately for each category
for (cat in levels(All$type)) {
  TPs <- sum(All$type == cat & All$model.class == cat)
  precision <- 100 * TPs / sum(All$model.class == cat)
  recall <- 100 * TPs / sum(All$type == cat)
  tmp <- data.frame(precision=precision, recall=recall)
  rownames(tmp) <- cat
  print(round(tmp, 2))
}

## full confusion matrix might be useful for discussion
print(table(All$type, All$model.class)) # rows = gold standard, columns = model output
