##  Author: Stefan Evert
## Purpose: Sample evaluation for Free Association Task 2

## load official test data
Test <- read.delim("FA/correlation_test.tbl")

## load model output (here for the FOO model)
Model <- read.delim("foo_output/task2.tbl")

## combine the test data and model output, matching on cue and target
##  - if your model output file uses different column names, change the by.y argument
All <- merge(Test, Model, by.x=c("cue","target"), by.y=c("cue","target"))
stopifnot(nrow(All) == nrow(Test)) # abort if model output doesn't have classifications for all cue-target pairs

## Pearson correlation coefficient and confidence interval (significant, but very low for FOO model)
cor.test(~ assoc + predict.assoc, data=All)

## explore scatterplot for indications of non-linearity etc.
plot(assoc ~ predict.assoc, data=All, xlab="Model prediction", ylab="True association")
## (some problems of the simplistic FOO model should be obvious here)

## if relationship may be non-linear, might also try rank correlation coefficient
##  - we recommend Kendall's tau, which can be interpreted in terms of the number of concordant pairs
##  - scale: 1 = all pairs of items are concordant, 0 = no correlation, -1 = all pairs are discordant
cor.test(~ assoc + predict.assoc, data=All, method="kendall")
