#!/usr/bin/perl
# -*-cperl-*-
###################################################################
###                                                             ###
###    Author: Stefan Evert                                     ###
###   Purpose: Sample evaluation for Free Association Task 3    ###
###                                                             ###
###################################################################

use warnings;
use strict;

use FileHandle;

die <<STOP unless @ARGV == 2;
Usage:  perl eval_task3.perl <gold_standard> <model_output>

For demonstration, type:
    perl eval_task3.perl FA/prediction_test.tbl foo_output/task3_t.score.txt
STOP

our ($gold_standard, $model_output) = @ARGV;

our %modelRank = (); 		# $modelRank{$cue}{$cand} = $rank
our @Ranks = ();		# ranks of correct targets in model suggestions

## read model output and generate database of rankings
my $modelFH = new FileHandle $model_output
  or die "Can't read file '$model_output': $!";
while (<$modelFH>) {
  chomp;
  my ($cue, @cand) = split /\t/;
  my $rank = 1;			# accept only up to 100 candidates for each cue word
  while (@cand and $rank <= 100) {
    $modelRank{$cue}{shift @cand} = $rank;
    $rank++;
  }
}
$modelFH->close;

## now process the gold standard file
my $goldFH = new FileHandle $gold_standard
  or die "Can't read file '$gold_standard': $!";
$goldFH->getline;		# skip table header
while (<$goldFH>) {
  chomp;
  my ($cue, $target) = split /\t/;
  warn "!! model makes no suggestions for cue '$cue'\n"
    unless exists $modelRank{$cue};
  my $rank = $modelRank{$cue}{$target} || 100; # set rank to 100 if not found among suggestions
  push @Ranks, $rank;
}
$goldFH->close;

## compute various evaluation statistics
my $n = @Ranks;			# number of items in test set
my $avg_rank = sum(@Ranks) / $n;
printf "Average rank: %5.1f\n", $avg_rank;

foreach my $cutoff (qw(1 3 5 10 15 20 50)) {
  my $among_first = grep {$_ <= $cutoff} @Ranks;
  printf " - among first %2d suggestions for %5.2f%s of cues\n", $cutoff, 100 * $among_first / $n, '%';
}

## ----- helper function -----
sub sum {
  my $sum = 0;
  foreach (@_) { $sum += $_ }
  return $sum;
}

