#!/usr/bin/perl -w -I/Users/severt/UCS/System/perl/lib
# -*-cperl-*-
###################################################################
###                                                             ###
###    Author: Stefan Evert                                     ###
###   Purpose: Sample participant for free association task - The FOO system (first-order only)
###                                                             ###
###################################################################
$| = 1;

use strict;
use warnings;

use UCS;
use UCS::File;
use UCS::DS::Memory;

## load database of first-order associations
print "Loading database ... ";
our $DB = new UCS::DS::Memory "lexsem08_first_order_associations.ds.gz";
print "indexing ... ";
our $Cue_index = $DB->dict("l1"); # construct index for cue terms
print $DB->size, " entries\n";

foreach my $AM (qw(log.likelihood t.score MI Dice)) {
  our $outfile = "foo_output/task3_$AM.txt";
  print "Writing $outfile ";
  ## open input and output files
  our $inFH = UCS::File::Open("FA/prediction_test.tbl");
  our $outFH = UCS::File::Open("> $outfile");
  ## skip header of input file (output file has no header)
  $inFH->getline;
  while (<$inFH>) {
    my ($cue) = split /\t/; # extract cue word from first column
    $DB->make_index("cues", $Cue_index->lookup($cue)); # faster than searching entire database every time
    $DB->activate_index("cues");
    $DB->sort("cand", "-am.$AM", "l2");	# break ties by alphabetic ordering
    $DB->activate_index("cand");
    my $n_cand = $DB->size;	# number of candidates
    $n_cand = 100 if $n_cand > 100; # system may suggest up to 100 candidates
    print $outFH "$cue";
    foreach my $i (1 .. $n_cand) {
      print $outFH "\t", $DB->cell("l2", $i);
    }
    print $outFH "\n";
    $DB->activate_index();
    print "." if ($. % 10) == 0;
  }
  $inFH->close;
  $outFH->close;
  print " ok\n";
}
